/***********************************************************************

Vorosketch User Distances Demo by Herman Haverkort, 27 September 2022

************************************************************************

Copyright 2022 Herman Johannes Haverkort

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This file is designed to be used with Vorosketch v0.26 BETA; it might 
work with later versions too. New versions may be available from 
http://herman.haverkort.net.


***********************************************************************/


#define MAKEUSERDISTANCE \
if (dname == "mydistance") return new MyDistance();\
if (dname == "wave") return new WaveDistance();\
if (dname == "wiggle") return new WiggleDistance();


class MyDistance: public PointDistance
{
    public:
  virtual const string description() const
  {
    return "mydistance";
  }

  virtual DOUBLE distanceToPointSite(const Point& p, const PointSite* s) const
  {
    return (p - s->location).length();
  }
};


class WaveDistance: public SetDistance
{
    public:
  virtual const string description() const
  {
    return "wave";
  }

  virtual DOUBLE distanceToPointSite(const Point& p, const PointSite* s) const
  {
    DOUBLE d = EuclideanDistance().distanceToPointSite(p, s);
    return(1.0 + (sin(d * 10 - PI * 0.5) - 1.0) * pow(0.1, d) * 0.5);
  }

  virtual DOUBLE distanceToSegmentSite(const Point& p, const SegmentSite* s) const
  {
    DOUBLE d = EuclideanDistance().distanceToSegmentSite(p, s);
    return(1.0 + (sin(d * 10 - PI * 0.5) - 1.0) * pow(0.1, d) * 0.5);
  }
};


class WiggleDistance: public RootedVectorDistance
{
    public:
  virtual const string description() const
  {
    return "wiggle";
  }

  virtual DOUBLE distanceToRootedVectorSite(const Point& p, const RootedVectorSite* s) const
  {
    Point v = p - s->location;
    DOUBLE b = 0;
    if (!v.inOrigin())
    {
      // rotate such that s->direction points in direction 0
      Point n(s->direction * v, s->normal * v);
      b = atan2(n.y, n.x);
    }
    DOUBLE wiggle = 0;
    for (int i = 4; i <= 32; i *= 4) wiggle += sin(b * i + HALFPI/i)/i;
    for (int i = 8; i <= 32; i *= 4) wiggle -= sin(b * i + HALFPI/i)/i;
    return wiggle;
  }

  virtual void boundDistanceToRootedVectorSite(
    const Point& centre, DOUBLE halfwidth, const RootedVectorSite* s, Range& range
  ) const
  {
    range.set(-0.5,0.5);
  }
};

